unit hiUseActiveX;

interface

uses Win,Share,ActiveKOL,KOLComObj,Kol,ActiveScp,ActiveX,err,Windows;

type

  TScriptLanguage = (slVBScript, slJScript);

  PMyOleCtl = ^TMyOleCtl;
  TMyOleCtl = object(TOleCtl)
   protected
    procedure InitControlData; virtual;
   public
    function GetEventTypeInfo: ITypeInfo;
  end;

  PMyScriptSite = ^TMyScriptSite;
  TMyScriptSite = class(TInterfacedObject,IActiveScriptSite,IActiveScriptSiteWindow)
   private
    FEngine: IActiveScript;
    FParser: IActiveScriptParse;
    FControl: IUnknown;
    procedure SetControlIUnknown(pUnk: IUnknown);
   public
    destructor Destroy; override;
    procedure CreateScriptEngine(Language: TScriptLanguage);
    procedure CloseScriptEngine;
    procedure AddCode(const Code:string);
    property ControlIUnknown: IUnknown write SetControlIUnknown;
   protected
    { IActiveSriptSite }
    function GetLCID(out plcid: LCID): HResult; stdcall;
    function GetItemInfo(pstrName: LPCOLESTR; dwReturnMask: DWORD;
      out ppiunkItem: IUnknown; out ppti: ITypeInfo): HResult; stdcall;
    function GetDocVersionString(out pbstrVersion: WideString): HResult; stdcall;
    function OnScriptTerminate(var pvarResult: OleVariant;
      var pexcepinfo: EXCEPINFO): HResult; stdcall;
    function OnStateChange(ssScriptState: SCRIPTSTATE): HResult; stdcall;
    function OnScriptError(const FScriptError: IActiveScriptError): HResult; stdcall;
    function OnEnterScript: HResult; stdcall;
    function OnLeaveScript: HResult; stdcall;
    { IActiveSriptSiteWindow }
    function GetWindow(out phwnd: HWND): HResult; stdcall;
    function EnableModeless(fEnable: BOOL): HResult; stdcall;
  end;

  TMyEventHandler = class;
  THIUseActiveX = class(THIWin)
   private
    FSite: TMyScriptSite;
    FEventHandler: TMyEventHandler;
    FEventConnection: longint;
   public
    _prop_CLSID: string;
    _event_onEvent: THI_Event;
    constructor Create(Parent:PControl);
    procedure Init; override;
    procedure _work_doExecute(var Data:TData; Index:word);
  end;

  TMyEventHandler = class(TObject, IUnknown, IDispatch)
  private
    FMe: THIUseActiveX;
    FTypeInfo: ITypeInfo;
    FEventIID: TGUID;
  protected
    { IUnknown }
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    { IDispatch }
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;  NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
  public
    constructor Create(Me: THIUseActiveX; EventTypeInfo: ITypeInfo);
    property IID: TGUID read FEventIID;
  end;

implementation

var
  NewCLSID:string;
  ScriptCLSIDs: array[TScriptLanguage] of TGUID;

procedure TMyOleCtl.InitControlData;
var data: ^TControlData2;
begin
  new(data);
  with data^ do begin
    if CLSIDFromString(StringToOleStr(NewCLSID), ClassID)<>S_OK then
      raise Exception.Create(e_InvalidPointer,'ActiveX control not registered.');
    Version := 401;
  end;
  ControlData := pointer(data);
end;

function TMyOleCtl.GetEventTypeInfo;
var pdisp:IDispatch; nti:integer; pti:ITypeInfo;
    plib:ITypeLib; pattr:PTypeAttr; nFlags:integer; hRef:HRefType;
begin
  Result := nil; pdisp := OleObject;
  pdisp.GetTypeInfoCount(nti); if nti=0 then exit;
  pdisp.GetTypeInfo(0, LOCALE_SYSTEM_DEFAULT, pti);
  pti.GetContainingTypeLib(plib, nti); pti := nil;
  plib.GetTypeInfoOfGuid(FControlData^.ClassID, pti);
  pti.GetTypeAttr(pattr);
  for nti:=0 to pattr.cImplTypes do begin
    pti.GetImplTypeFlags(nti, nFlags);
    if ((nFlags and IMPLTYPEFLAG_FDEFAULT)<>0) and ((nFlags and IMPLTYPEFLAG_FSOURCE)<>0) then begin
      pti.GetRefTypeOfImplType(nti, hRef);
      pti.GetRefTypeInfo(hRef, Result);
      break;
    end;
  end;
  pti.ReleaseTypeAttr(pattr);
end;

function NewMyOleCtl(AParent:PControl; CLSID:string): PMyOleCtl;
begin
  NewCLSID := CLSID;
  New(Result, CreateParented(AParent));
end;

{ THIUseActiveX }

constructor THIUseActiveX.Create;
begin
  inherited Create(Parent);
  FSite := TMyScriptSite.Create;
end;

procedure THIUseActiveX.Init;
var pCtl: PMyOleCtl;
begin
  try
    pCtl := NewMyOleCtl(FParent, _prop_CLSID); Control := pCtl;
    FSite.CreateScriptEngine(slVBScript);
    FSite.ControlIUnknown := pCtl.OleObject;
    FEventHandler := TMyEventHandler.Create(Self, pCtl.GetEventTypeInfo);
    InterfaceConnect(pCtl.OleObject, FEventHandler.IID, FEventHandler, FEventConnection);
  except
    on E:Exception do begin
      Control := NewLabel(FParent, E.Message);
      Control.Color := $FFFFFF;
      Control.Font.Color := $FF;
    end;
  end;
  //inherited Init;
  with Control{$ifndef F_P}^{$endif} do begin
    SetPosition(_prop_Left,_prop_Top);
    SetSize(_prop_Width,_prop_Height);
    Enabled := _prop_Enabled;
    Visible := _prop_Visible;
  end;
  inherited Init;
end;

procedure THIUseActiveX._work_doExecute;
begin
  FSite.AddCode(ToString(Data));
end;

{ TMyScriptSite }

procedure TMyScriptSite.SetControlIUnknown;
begin
  FControl := pUnk;
  FEngine.AddNamedItem('Control', SCRIPTITEM_GLOBALMEMBERS);
end;

destructor TMyScriptSite.Destroy;
begin
  CloseScriptEngine;
end;

procedure TMyScriptSite.CreateScriptEngine;
begin
  CloseScriptEngine;
  FEngine := CreateComObject(ScriptCLSIDs[Language]) as IActiveScript;
  FParser := FEngine as IActiveScriptParse;
  FEngine.SetScriptSite(Self);
  FParser.InitNew;
end;

procedure TMyScriptSite.CloseScriptEngine;
begin
  FParser := nil;
  if FEngine <> nil then FEngine.Close;
  FEngine := nil;
end;

procedure TMyScriptSite.AddCode;
var Result: OleVariant; ExcepInfo: TEXCEPINFO;
begin
  if FParser<>nil then
    FParser.ParseScriptText(StringToOleStr(Code), nil, nil, nil, 0, 0, 0, Result, ExcepInfo);
end;

{ IActiveScriptSite implementation }

function TMyScriptSite.GetDocVersionString(out pbstrVersion: WideString): HResult;
begin
  Result := E_NOTIMPL;
end;

function TMyScriptSite.GetItemInfo(pstrName: LPCOLESTR; dwReturnMask: DWORD;
  out ppiunkItem: IUnknown; out ppti: ITypeInfo): HResult;
begin
  if @ppiunkItem <> nil then Pointer(ppiunkItem) := nil;
  if @ppti <> nil then Pointer(ppti) := nil;
  if ((dwReturnMask and SCRIPTINFO_IUNKNOWN) <> 0) and (pstrName='Control') then
    ppiunkItem := FControl;
  Result := S_OK;
end;

function TMyScriptSite.GetLCID(out plcid: LCID): HResult;
begin
  plcid := GetSystemDefaultLCID;
  Result := S_OK;
end;

function TMyScriptSite.OnEnterScript: HResult;
begin
  Result := S_OK;
end;

function TMyScriptSite.OnLeaveScript: HResult;
begin
  Result := S_OK;
end;

function TMyScriptSite.OnScriptError;
var ei: EXCEPINFO;
begin
  Result := S_OK;
  if FScriptError = nil then exit;
  FScriptError.GetExceptionInfo(ei);
  if @ei.pfnDeferredFillIn <> nil then ei.pfnDeferredFillIn(@ei);
  MessageBox(ReadHandle,PChar(string(ei.bstrDescription)),PChar(string(ei.bstrSource)),MB_OK+MB_ICONSTOP);
end;

function TMyScriptSite.OnScriptTerminate(var pvarResult: OleVariant;
  var pexcepinfo: EXCEPINFO): HResult;
begin
  Result := S_OK;
end;

function TMyScriptSite.OnStateChange(ssScriptState: SCRIPTSTATE): HResult;
begin
  Result := S_OK;
end;

{ IActiveScriptSiteWindow implementation }

function TMyScriptSite.EnableModeless(fEnable: BOOL): HResult;
begin
  Result := S_OK;
end;

function TMyScriptSite.GetWindow(out phwnd: HWND): HResult;
begin
  phwnd := ReadHandle;
  Result := S_OK;
end;

{ TMyEventHandler implementation }

constructor TMyEventHandler.Create;
var pattr:PTypeAttr;
begin
  FMe := Me; FTypeInfo := EventTypeInfo;
  FTypeInfo.GetTypeAttr(pattr);
  FEventIID := pattr.guid;
  FTypeInfo.ReleaseTypeAttr(pattr);
end;

function TMyEventHandler.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if GetInterface(IID, Obj) then begin
    Result := S_OK;
    Exit;
  end;
  if IsEqualIID(IID, FEventIID) then begin
    GetInterface(IDispatch, Obj);
    Result := S_OK;
    Exit;
  end;
  Result := E_NOINTERFACE;
end;

function TMyEventHandler._AddRef: Integer;
begin
  Result := PMyOleCtl(FMe.Control).OleObject._AddRef;
end;

function TMyEventHandler._Release: Integer;
begin
  Result := PMyOleCtl(FMe.Control).OleObject._Release;
end;

function TMyEventHandler.GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount,
  LocaleID: Integer; DispIDs: Pointer): HResult;
begin
  Result := DISP_E_UNKNOWNNAME
end;

function TMyEventHandler.GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult;
begin
  Pointer(TypeInfo) := nil;
  Result := E_NOTIMPL;
end;

function TMyEventHandler.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Count := 0;
  Result := S_OK;
end;

function TMyEventHandler.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
var s: string; p:TDispParams absolute Params; name: PWideChar;
begin
  FTypeInfo.GetDocumentation(DispID,@name,nil,nil,nil);
  s := OleStrToString(name);//+'('+int2str(p.cArgs)+')';
  SysFreeString(name);
  _hi_OnEvent(FMe._event_onEvent, s);
  Result := S_OK;
end;

{ init }

procedure InitCLSIDs;
const
  NULL_GUID: TGUID = '{00000000-0000-0000-0000-000000000000}';
  ScriptProgIDs: array[TScriptLanguage] of PWideChar = ('VBScript','JScript');
var
  L: TScriptLanguage;
begin
  for L := Low(TScriptLanguage) to High(TScriptLanguage) do
    if CLSIDFromProgID(ScriptProgIDs[L], ScriptCLSIDs[L]) <> S_OK
      then ScriptCLSIDs[L] := NULL_GUID;
end;

initialization

OleInit;
InitCLSIDs;

end.
